#! /usr/bin/env python
# vim: set fileencoding=utf-8 :
#
# Copyright (C), 2015, Christopher Swingley <cswingle@swingleydev.com>
# and licensed under the terms of the creative commons
# Attribution-ShareAlike 4.0 International license
#
usage = """Process the yearly GHCN-Daily data files and produce INSERT
lines appropriate for mason:noaa.ghcnd_obs.

Yearly files downloaded from:

    http://www1.ncdc.noaa.gov/pub/data/ghcn/daily/by_year/

station_list file contains a list of station codes, one line per station.
"""
import argparse
import sys
import gzip
import re

parser = argparse.ArgumentParser(
    description=usage,
    formatter_class=argparse.RawDescriptionHelpFormatter)
group = parser.add_mutually_exclusive_group()
group.add_argument("-v", "--verbose", action="store_true", dest="verbose",
                   default=True, help="be more verbose")
group.add_argument("-q", "--quiet", action="store_false", dest="verbose",
                   default=True, help="be quiet")

parser.add_argument("ghcnd_file",
                    help="ghcn-daily_by_year filename (2011.csv.gz)")
parser.add_argument("station_list", nargs="?",
                    help="limit insertions to station IDs in this file")
parser.add_argument("outfile", nargs="?", type=argparse.FileType('w'),
                    default=sys.stdout)

args = parser.parse_args()


def str_nuller(s):
    """ Nullify or quotify strings """
    if s is None or len(s) == 0:
        return 'NULL'
    else:
        return "'{0}'".format(s)


def str_blanker(s):
    """ blank quotify or quotify strings """
    if s is None or len(s) == 0:
        return "''"
    else:
        return "'{0}'".format(s)


def int_nuller(s):
    """ Nullify or integerify strings """
    if s is None or len(s) == 0:
        return 'NULL'
    else:
        return int(s)

stations_to_include = []
if args.station_list is not None:
    with open(args.station_list, 'r') as inf:
        for line in inf.readlines():
            line = line.strip()
            stations_to_include.append(line)

if args.ghcnd_file[-2:] == 'gz':
    inf = gzip.open(args.ghcnd_file, 'rb')
else:
    inf = open(args.ghcnd_file, 'r')

for line in inf:
    line = line.strip().decode()
    (station_id, dte, variable, raw_value,
     meas_flag, qual_flag, source_flag, time_of_obs) = line.split(',')
    if len(stations_to_include) == 0 or station_id in stations_to_include:
        (station_id, variable, time_of_obs) = [str_nuller(x)
                                               for x in
                                               (station_id, variable,
                                                time_of_obs)]
        (meas_flag, qual_flag, source_flag) = [str_blanker(x)
                                               for x in
                                               (meas_flag, qual_flag,
                                                source_flag)]
        (yyyy, mm, dd) = re.search('([0-9]{4})([0-9]{2})([0-9]{2})',
                                   dte).groups()
        if time_of_obs != 'NULL':
            time_of_obs = int(time_of_obs[1:3])
            if time_of_obs > 23:
                time_of_obs = 24 - time_of_obs
        raw_value = int_nuller(raw_value)
        if raw_value != 'NULL':
            print(("INSERT INTO ghcnd_obs (station_id, dte, variable, "
                   "raw_value, meas_flag, qual_flag, source_flag, "
                   "time_of_obs) VALUES ({0}, '{1}-{2}-{3}', {4}, {5}, {6}, "
                   "{7}, {8}, {9});".format(
                       station_id, yyyy, mm, dd, variable, raw_value,
                       meas_flag, qual_flag, source_flag, time_of_obs)))
